import { aws_ec2 as ec2 } from "aws-cdk-lib";
import { Construct } from "constructs";
/** Common properties */
export interface CodePipelineBitbucketBuildResultReporterProps {
    /**
     * The VPC in which to run the status reporter.
     */
    readonly vpc?: ec2.VpcAttributes;
    /**
     * The SSM parameter (SecureString) name that contains the BitBucket access token for reporting build statuses.
     * @default BITBUCKET_UPDATE_BUILD_STATUS_TOKEN
     */
    readonly bitbucketAccessTokenName?: string;
    /**
     * The BitBucket server address.
     */
    readonly bitbucketServerAddress: string;
}
/** A CDK construct for reporting CodePipeline build statuses to a BitBucket server using BitBucket REST API.
 * You need to configure SSM parameter `BITBUCKET_UPDATE_BUILD_STATUS_TOKEN` before using the construct.
 *
 * @stability stable
 */
export declare class CodePipelineBitbucketBuildResultReporter extends Construct {
    constructor(scope: Construct, id: string, props: CodePipelineBitbucketBuildResultReporterProps);
}
