import { ContainerIterator, initContainer } from "../ContainerBase/index";
import SequentialContainer from './Base/index';
export declare class DequeIterator<T> extends ContainerIterator<T> {
    private node;
    private readonly size;
    private readonly getElementByPos;
    private readonly setElementByPos;
    constructor(index: number, size: () => number, getElementByPos: (pos: number) => T, setElementByPos: (pos: number, element: T) => void, iteratorType?: 'normal' | 'reverse');
    get pointer(): T;
    set pointer(newValue: T);
    pre(): this;
    next(): this;
    equals(obj: DequeIterator<T>): boolean;
}
declare class Deque<T> extends SequentialContainer<T> {
    private first;
    private curFirst;
    private last;
    private curLast;
    private bucketNum;
    private readonly bucketSize;
    private map;
    constructor(container?: initContainer<T>, bucketSize?: number);
    private reAllocate;
    private getElementIndex;
    clear(): void;
    front(): T | undefined;
    back(): T | undefined;
    begin(): DequeIterator<T>;
    end(): DequeIterator<T>;
    rBegin(): DequeIterator<T>;
    rEnd(): DequeIterator<T>;
    pushBack(element: T): void;
    popBack(): void;
    /**
     * Push the element to the front.
     */
    pushFront(element: T): void;
    /**
     * Remove the first element.
     */
    popFront(): void;
    forEach(callback: (element: T, index: number) => void): void;
    getElementByPos(pos: number): T;
    setElementByPos(pos: number, element: T): void;
    insert(pos: number, element: T, num?: number): void;
    /**
     * Remove all elements after the specified position (excluding the specified position).
     */
    cut(pos: number): void;
    eraseElementByPos(pos: number): void;
    eraseElementByValue(value: T): void;
    eraseElementByIterator(iter: DequeIterator<T>): DequeIterator<T>;
    find(element: T): DequeIterator<T>;
    reverse(): void;
    unique(): void;
    sort(cmp?: (x: T, y: T) => number): void;
    /**
     * Remove as much useless space as possible.
     */
    shrinkToFit(): void;
    [Symbol.iterator](): Generator<T, void, unknown>;
}
export default Deque;
